<?php
/*
 * Web service: COPA Manager
 * description: this provides a set of functions for working with COPA technology
 * functions.
 * Response XML-based message:
 *		<Response>
 *			<!-- This part depends on the operation -->
 *			<OperationInfo>
 *				<Status status="value"/>				<!-- values: success or fail -->
 *				<ErrorMessage message="description"/>	<!-- included if there is any error -->
 *			</OperationInfo>
 *		</Response>
 *
 * Available functions:
 * 		- initCopaVirtualViewNav
 *			+ Description: provides an entry the capacity to includes virtual view configurations
 *			+ Parameters:
 *				* oper = initCopaVirtualViewNav
 *				* dn = <DN in base 64>
 *			+ Usage: copaManager.php?oper=initCopaVirtualViewNav&dn=ZGM9ZXM=
 * 			+ Response XML-based message:
 *				<Response>
 *					<OperationInfo>...</OperationInfo>
 *				</Response>
 *
 * 		- listVirtualViews
 *			+ Description: returns a list containing virtual view configurations that we can use at the selected dn
 *			+ Parameters:
 *				* oper = listVirtualViews
 *				* dn = <DN in base 64>
 *			+ Usage: copaManager.php?oper=listVirtualViews&dn=ZGM9ZXM=
 * 			+ Response XML-based message:
 *				<Response>
 *					<VirtualViews>
 *						<VirtualView dn="DN">						<!-- It can includes one or more <VirtualView> -->
 *							<Name value="NAME"/>
 *							<Default value="IS_THE_DEFAULT_VV"/>	<!-- values: true or false -->
 *							<Description value="DESCRIPTION"/> 		<!-- it can includes one or more descriptions -->
 *						</VirtualView>
 *					</VirtualViews>
 *					<OperationInfo>...</OperationInfo>
 *				</Response>
 *
 * 		- listAreas
 *			+ Description: returns a list containing the children areas of an specific area.
 *			+ Parameters:
 *				* oper = listAreas
 *				* dnVV = <DN of the virtual view in base 64>
 *				* copaCodeArea = copa code of the parent area. This parameter is optional. If it isn't included, this function returns areas of the root level.
 *			+ Usage: copaManager.php?oper=listAreas&dnVV=ZGM9dnYxLGRjPXVuaXYsZGM9bG9jYWw=
 * 			+ Response XML-based message:
 *				<Response>
 *					<Areas> <!-- It includes one or more <Area> elements -->
 *						<Area dn="DN_OF_THIS_AREA" copaCode="COPA_CODE">
 *							<Name value="NAME_OF_THE_AREA"/>
 *							<Description value="DESCRIPTION_OF_THE_AREA"/>
 *						</Area>
 *					</Areas>
 *					<OperationInfo>...</OperationInfo>
 *				</Response>
 *
 * 		- getArea
 *			+ Description: returns the information of an area. There are two cases in which an area can be obtained:
 *			+ A) Get an area which is referenced by the DN of a virtual view and the copa code of itself.
 *				+ Parameters:
 *					* oper = getArea
 *					* dnVV = <DN of the virtual view in base 64>
 *					* copaCodeArea = copa code of the requested area
 *					* includeVV = true or false. It sets if the response has to return the information about the virtual view of the area.
 *				+ Usage: copaManager.php?oper=getArea&dnVV=ZGM9dnYxLGRjPXVuaXYsZGM9bG9jYWw=&copaCodeArea=a01
 *			+ B) Get an area which is referenced by an URN.
 *				+ Parameters:
 *					* oper = getArea
 *					* urnArea = <URN of the area in base 64>
 *					* includeVV = true or false. It sets if the response has to return the information about the virtual view of the area.
 *				+ Usage: copaManager.php?oper=getArea&urnArea=dXJuOm1hY2U6cmVkaXJpcy5lczp2aXJ0dWFsdmlldzpjYXRyZToyMDA1MDEwMS0xLjA6YTAy
 * 			+ Response XML-based message:
 *				<Response>
 *					<Areas> <!-- It includes one <Area> element -->
 *						<Area dn="DN_OF_THIS_AREA" copaCode="COPA_CODE">
 *							<Name value="NAME_OF_THE_AREA"/>
 *							<Description value="DESCRIPTION_OF_THE_AREA"/>
 *							<RelatedArea urn="URN_OF_A_RELATED_AREA"/> <!-- One or more <RelatedArea> elements -->
 *						</Area>
 *					</Areas>
 *					<OperationInfo>...</OperationInfo>
 *				</Response>
 *
 * 		- listResources
 *			+ Description: returns the list of resources which are associated to an area.
 *			+ Parameters:
 *				* oper = listResources
 *				* dnbase = <DN base for the search in base 64>
 *				* dnVV = <DN of the virtual view in base 64>
 *				* copaCodeArea = copa code of the area
 *			+ Usage: copaManager.php?oper=listResources&dnbase=ZGM9dW5pdixkYz1sb2NhbA==&dnVV=ZGM9dnYxLGRjPXVuaXYsZGM9bG9jYWw=&copaCodeArea=a01
 * 			+ Response XML-based message:
 *				<Response>
 *					<Resources> <!-- it contains zero or more <Entry> elements -->
 *						<Entry dn="DN">
 *							<Attribute name="NAME"> <!-- list of attributes -->
 *								<AttributeValue lang="LANGUAGE" value="VALUE"/> <!-- If the value of the attribute is in an specific language, <AttributeValue> entries will include the attribute 'lang'. Also, if the attribute is binary, it will encode the value in base 64. -->
 *							</Attribute>
 *						</Entry>
 *					</Resources>
 *					<OperationInfo>...</OperationInfo>
 *				</Response>
 *
 */

require("../LdapManager.php");
require("../LdapConf.php");
require("../LdapSchema.php");
require("../LdapObjClassDef.php");

function xmlParse($valueAttr) {
    $valueAttr=ereg_replace("&","&amp;",$valueAttr); 
    $valueAttr=ereg_replace("<","&lt;",$valueAttr);
    $valueAttr=ereg_replace(">","&gt;",$valueAttr);
    $valueAttr=ereg_replace('"',"&quot;",$valueAttr);
    $valueAttr=ereg_replace("'","&apos;",$valueAttr);
	return $valueAttr;
}

function getFilterAreas($copaCodeAttr,$area='') {
	if (strcmp($area,"")==0) {
		return '('.$copaCodeAttr.'=a*)(!('.$copaCodeAttr.'=a*b*))';
	}
	else if (strcmp($area,"*")==0) {
		return '('.$copaCodeAttr.'=a*)';
	}
	else {
		ereg("(([a-z])([0-9])+)+",$area,$tokens);
		$letter=$tokens[2];
		$numLetter=ord($letter);
   		$letter=chr($numLetter+1); 
   		$nextLetter=chr($numLetter+2); 
		return '('.$copaCodeAttr.'='.$area.$letter.'*)(!('.$copaCodeAttr.'='.$area.$letter.'*'.$nextLetter.'*))';
	}
}

function initCopaVirtualViewNav($lm,$dn) {
   	$entry=$lm->getEntry($dn);
    if (array_key_exists('lang',$_REQUEST)) {
	    $entry->selectLang($_REQUEST['lang']);
    }
    $objClassList=$entry->getObjectClassList();
    array_push($objClassList,"copaVirtualViewNav");
    $attrList=$entry->getAttributeValues();
	if ($lm->replaceEntry($dn,array_keys($attrList),$objClassList,$attrList)) {
		$success=true;
	}
}

function echoVirtualViewData($elemVV) {
   	echo "  <VirtualView dn=\"".$elemVV->getDN()."\">\n";
   	echo "     <Name value=\"".$elemVV->getSingleAttribute("copaName")."\" />\n";
   	$descs=$elemVV->getAttribute("description");
   	foreach ($descs->getValues() as $desc) {
	   	echo "     <Description value=\"".$desc."\" />\n";
   	}
   	echo "  </VirtualView>\n";
}

function listVirtualViews($lm,$dn) {
	$res="";
   	$res.="<VirtualViews>\n";
   	$entry=$lm->getEntry($dn);
    if (array_key_exists('lang',$_REQUEST)) {
	    $entry->selectLang($_REQUEST['lang']);
    }
   	// Add the default virtual view
   	$defVVdn=$entry->getSingleAttribute("copaDefVvNav");
   	$defVV=$lm->getEntry($defVVdn);
   	$res.="  <VirtualView dn=\"".$defVV->getDN()."\">\n";
   	$res.="     <Name value=\"".$defVV->getSingleAttribute("copaName")."\" />\n";
   	$res.="     <Default value=\"true\" />\n";
   	$descs=$defVV->getAttribute("description");
   	foreach ($descs->getValues() as $desc) {
	   	$res.="     <Description value=\"".$desc."\" />\n";
   	}
   	$res.="  </VirtualView>\n";
	// Add not default virtual views
   	$listVVdn=$entry->getAttribute("copaVvNav");
   	if (strcmp($listVVdn,"")!=0) {
   		foreach ($listVVdn->getValues() as $elemVVdn) {
	   		$elemVV=$lm->getEntry($elemVVdn);
	    	if (array_key_exists('lang',$_REQUEST)) {
			    $elemVV->selectLang($_REQUEST['lang']);
	    	}
   			echoVirtualViewData($elemVV);
   		}
   	}
   	$res.="</VirtualViews>\n";
	return $res;
}

function listAreas($lm,$dnVV,$copaCode) {
	$res="";
    $res.="<Areas>";
    $entryVV=$lm->getEntry($dnVV);
    if (array_key_exists('lang',$_REQUEST)) {
	    $entryVV->selectLang($_REQUEST['lang']);
    }
    $success=true;
        	
    $copaCodeAttr=$entryVV->getSingleAttribute("copaCodeAttr");
    $copaPrintAttr=$entryVV->getSingleAttribute("copaPrintAttr");
    $filterAreas=getFilterAreas($copaCodeAttr,$copaCode);
    $filterObj='';
    $copaAreaOCNAttr=$entryVV->getAttribute("copaAreaObjectClassName");
    foreach($copaAreaOCNAttr->getValues() as $areaObj) {
    	$filterObj.='(objectClass='.$areaObj.')';
    }
    $filter='(&'.$filterAreas.$filterObj.')';
    $areas=array();
    $bases=$entryVV->getAttribute("copaClassifBase");
    foreach($bases->getValues() as $classifDnBase) {
    	$listEntries=$lm->search($classifDnBase,$filter,false);
    	foreach($listEntries as $entry) {
		    if (array_key_exists('lang',$_REQUEST)) {
	    		$entry->selectLang($_REQUEST['lang']);
    		}
    		$nameArea=$entry->getSingleAttribute($copaPrintAttr);
    		$dnArea=$entry->getDN();
    		$copaCode=$entry->getSingleAttribute($copaCodeAttr);
    		$res.='<Area dn="'.xmlParse($dnArea).'" copaCode="'.xmlParse($copaCode).'">';
    		$res.='<Name value="'.xmlParse($nameArea).'" />';
    		$descArea=$entry->getAttribute("description");
    		foreach ($descArea->getValues() as $descValue) {
    			$res.='<Description value="'.xmlParse($descValue).'" />';
    		}
    		$res.='</Area>';
    	}
    }
        	
    $res.="</Areas>";
    
    return $res;
}

function getAreaByURN($lm,$urnArea,$includeVV) {
	global $LDAP_DN_BASE;
	
	$tokensUrn=explode(":",$urnArea);
	$areaCode=array_pop($tokensUrn);
	$prefixVV=implode(":",$tokensUrn);
	$filter="(copaPrefix=".$prefixVV.")";
	$listEntries=$lm->search($LDAP_DN_BASE,$filter,false);
	if (count($listEntries)>0) {
		$entryVV=$listEntries[0];
	    if (array_key_exists('lang',$_REQUEST)) {
		    $entryVV->selectLang($_REQUEST['lang']);
	    }
		return getArea($lm,$entryVV->getDN(),$areaCode,$includeVV);
	}
	else {
		return false;
	}
}

function getArea($lm,$dnVV,$copaCodeRaw,$includeVV) {
    echo "<Areas>\n";
    $entryVV=$lm->getEntry($dnVV);
    if (array_key_exists('lang',$_REQUEST)) {
	    $entryVV->selectLang($_REQUEST['lang']);
    }

    $copaCodeAttr=$entryVV->getSingleAttribute("copaCodeAttr");
    $copaPrintAttr=$entryVV->getSingleAttribute("copaPrintAttr");
    $bases=$entryVV->getAttribute("copaClassifBase");
    $copaAreaOCNAttr=$entryVV->getAttribute("copaAreaObjectClassName");
    
    $listCopaCodes=explode(",",$copaCodeRaw);
    foreach ($listCopaCodes as $copaCode) {
	    $filterAreas='('.$copaCodeAttr.'='.$copaCode.')';
	    $filterObj='';
	    foreach($copaAreaOCNAttr->getValues() as $areaObj) {
	    	$filterObj.='(objectClass='.$areaObj.')';
	    }
	    $filter='(&'.$filterAreas.$filterObj.')';
	    $areas=array();
	    foreach($bases->getValues() as $classifDnBase) {
	    	$listEntries=$lm->search($classifDnBase,$filter,false);
	    	foreach($listEntries as $entry) {
			    if (array_key_exists('lang',$_REQUEST)) {
				    $entry->selectLang($_REQUEST['lang']);
			    }
	    		$nameArea=$entry->getSingleAttribute($copaPrintAttr);
	    		$dnArea=$entry->getDN();
	    		$copaCode=$entry->getSingleAttribute($copaCodeAttr);
	    		echo '<Area dn="'.xmlParse($dnArea).'" copaCode="'.xmlParse($copaCode).'">';
	    		echo '<Name value="'.xmlParse($nameArea).'" />';
	    		$descArea=$entry->getAttribute("description");
	    		foreach ($descArea->getValues() as $descValue) {
	    			echo '<Description value="'.xmlParse($descValue).'" />';
	    		}
	    		$relatedAreas=$entry->getAttribute($entryVV->getSingleAttribute("copaRelatedAttr"));
	    		if (strcmp($relatedAreas,"")!=0) {
		    		foreach ($relatedAreas->getValues() as $relArea) {
		    			echo '<RelatedArea urn="'.$relArea.'" />';
	    			}
	    		}
	    		echo '</Area>';
	    	}
	    }
    }
        	
    echo "</Areas>\n";
    if ($includeVV==1) {
    	echoVirtualViewData($entryVV);
    }
    
    return true;
}

function listResources($lm,$dnbase,$dnVV,$copaCode) {
    echo "<Resources>\n";
    $entryVV=$lm->getEntry($dnVV);
    if (array_key_exists('lang',$_REQUEST)) {
	    $entryVV->selectLang($_REQUEST['lang']);
    }
    $success=true;
    $copaCodeResourceAttr=$entryVV->getSingleAttribute("copaCodeResourceAttr");
    $copaPrefix=$entryVV->getSingleAttribute("copaPrefix");
    $filter='('.$copaCodeResourceAttr.'='.$copaPrefix.":".$copaCode.')';
    $listEntries=$lm->search($dnbase,$filter,false);
    foreach($listEntries as $entry) {
	    if (array_key_exists('lang',$_REQUEST)) {
		    $entry->selectLang($_REQUEST['lang']);
	    }
    	echo $entry->getXML();
    }
    echo "</Resources>\n";
}

function setLinkResource($lm,$entryVV,$entryArea,$entryRes,$nameObjClass) {
    $neededAttr=$entryVV->getSingleAttribute("copaCodeResourceAttr");
	$oldattrs=$entryRes->getNameAttributes(false);
    	
    $objsentry=$entryRes->getObjectClassList()->getValues();
    if (!in_array($nameObjClass,$objsentry)) {
    	$objsentry[]=$nameObjClass;
    }
    	
    $attrsentry=$entryRes->getArrayAttributes();
    if (!array_key_exists($neededAttr,$attrsentry)) {
    	$attrsentry[$neededAttr]=array();
    }
    $attrsentry[$neededAttr][]=$entryVV->getSingleAttribute("copaPrefix").":".$entryArea->getSingleAttribute($entryVV->getSingleAttribute("copaCodeAttr"));
    	
    return $lm->replaceEntry($entryRes->getDN(),$oldattrs,$objsentry,$attrsentry);
}

function addLinkResource($lm,$dnVV,$dnArea,$dnRes,$nameObj) {
    $entryVV=$lm->getEntry($dnVV);
    $entryArea=$lm->getEntry($dnArea);
    $entryRes=$lm->getEntry($dnRes);

    $schema=new LdapSchema();
    $objs=$schema->getSchema(true);
    
    $availables=array();
    
    $neededAttr=$entryVV->getSingleAttribute("copaCodeResourceAttr");
    foreach ($objs as $obj) {
    	if (strcmp($obj->getType(),"Auxiliary")==0) {
    		$musts=$obj->getMustAttributes();
    		if (count($musts)==0) {
    			$mays=$obj->getMayAttributes();
    			if (in_array($neededAttr,$mays)) {
    				$availables[]=$obj;
    			}
    		}
    		else if (count($musts)==1) {
    			if (strcmp($musts[0],$neededAttr)==0) {
    				$availables[]=$obj;
    			}
    		}
    	}
    }
    if (count($availables)==0) { // We cannot do it!
    	return false;
    }
    else if (count($availables)==1) { // Yeah yeah yeah
    	return setLinkResource($lm,$entryVV,$entryArea,$entryRes,$availables[0]->getName());
    }
    else { // More possibilities
    	if (strcmp($nameObj,"")==0) {
			echo "<ObjectClasses>";
			foreach ($availables as $obj) {
				echo $obj->getXML(true);
			}
			echo "</ObjectClasses>";
			return false;
    	}
    	else {
	    	return setLinkResource($lm,$entryVV,$entryArea,$entryRes,$nameObj);
    	}
    }
}


$lm=new LdapManager($LDAP_SERVER_NAME,$LDAP_SERVER_PORT);
$lm->setOption(LDAP_OPT_PROTOCOL_VERSION,3);
$lm->connectLdap($LDAP_USER,$LDAP_PASS);

$success=true;
header('Content-type: text/xml');
echo "<Response>\n";
if (array_key_exists('oper',$_REQUEST)||array_key_exists('oper',$_REQUEST)) {
	if (array_key_exists('oper',$_REQUEST)) {
    	$oper=$_REQUEST['oper'];
    }
    else {
    	$oper=$_REQUEST['oper'];
    }
    if (strcmp($oper,"initCopaVirtualViewNav")==0) { // Add a new entry
        if (array_key_exists('dn',$_REQUEST)) {
        	$dn=base64_decode($_REQUEST['dn']);
        	$success=initCopaVirtualViewNav($lm,$dn);
        }
    }
    else if (strcmp($oper,"addResource")==0) { // Link resource-area
        if (array_key_exists('dnVV',$_REQUEST)&&array_key_exists('dnArea',$_REQUEST)&&array_key_exists('dnResource',$_REQUEST)) {
        	$dnVV=base64_decode($_REQUEST['dnVV']);
        	$dnArea=base64_decode($_REQUEST['dnArea']);
        	$dnRes=base64_decode($_REQUEST['dnResource']);
        	$obj='';
        	if (array_key_exists('setObj',$_REQUEST)) {
        		$obj=base64_decode($_REQUEST['setObj']);
        	}
        	$success=addLinkResource($lm,$dnVV,$dnArea,$dnRes,$obj);
    	}
    }
    else if (strcmp($oper,"listVirtualViews")==0) { // List virtual views
        if (array_key_exists('dn',$_REQUEST)) {
        	$dn=base64_decode($_REQUEST['dn']);
        	echo listVirtualViews($lm,$dn);
        }
    }
    else if (strcmp($oper,"listAreas")==0) { // List areas
    	if (array_key_exists('dnVV',$_REQUEST)) {
    		$dnVV=base64_decode($_REQUEST['dnVV']);
    		if (array_key_exists('copaCodeArea',$_REQUEST)) {
	    		$copaCode=($_REQUEST['copaCodeArea']);
	    	}
	    	else {
	    		$copaCode='';
	    	}
    		echo listAreas($lm,$dnVV,$copaCode);
    	}
    }
    else if (strcmp($oper,"getArea")==0) { // Get area
    	if (array_key_exists('dnVV',$_REQUEST)&&array_key_exists('copaCodeArea',$_REQUEST)) {
    		$dnVV=base64_decode($_REQUEST['dnVV']);
    		$copaCode=($_REQUEST['copaCodeArea']);
    		$includeVV=0;
	    	if (array_key_exists('includeVV',$_REQUEST)) {
    			$includeVV=$_REQUEST['includeVV'];
	    	}
    		$success=getArea($lm,$dnVV,$copaCode,$includeVV);
    	}
    	else if (array_key_exists('urnArea',$_REQUEST)) {
    		$urnArea=base64_decode($_REQUEST['urnArea']);
    		$includeVV=0;
	    	if (array_key_exists('includeVV',$_REQUEST)) {
    			$includeVV=$_REQUEST['includeVV'];
	    	}
    		$success=getAreaByURN($lm,$urnArea,$includeVV);
    	}
    }
    else if (strcmp($oper,"listResources")==0) { // List resources
    	if (array_key_exists('dnbase',$_REQUEST)&&array_key_exists('dnVV',$_REQUEST)&&array_key_exists('copaCodeArea',$_REQUEST)) {
    		$dnbase=base64_decode($_REQUEST['dnbase']);
    		$dnVV=base64_decode($_REQUEST['dnVV']);
    		$copaCode=($_REQUEST['copaCodeArea']);
    		$success=listResources($lm,$dnbase,$dnVV,$copaCode);
    	}
    }
}
if ($success==true) {
    echo "<OperationInfo><Status status='success'/></OperationInfo>";
}
else {
    echo "<OperationInfo><Status status='fail'/><ErrorMessage message='".$lm->getErrorMessage()."'/></OperationInfo>";
}
echo "</Response>\n";
?>
